import { EventEmitter } from '@angular/core';
import { Observable } from 'rxjs';
import { EntityKey, ODataActionOptions, ODataEntityAnnotations, ODataEntityResource, ODataFunctionOptions, ODataNavigationPropertyResource, ODataOptions, ODataPropertyResource, ODataQueryArgumentsOptions, ODataQueryOptionsHandler, ODataResource, ODataSingletonResource } from '../resources';
import { ODataStructuredType } from '../schema';
import { ODataCollection } from './collection';
import { ModelOptions, ODataModelEvent, ODataModelField, ODataModelOptions, ODataModelRelation } from './options';
export declare class ODataModel<T> {
    static options: ModelOptions;
    static meta: ODataModelOptions<any>;
    static buildMeta<T>({ options, schema, }: {
        options?: ModelOptions;
        schema: ODataStructuredType<T>;
    }): void;
    _parent: [
        ODataModel<any> | ODataCollection<any, ODataModel<any>>,
        ODataModelField<any> | null
    ] | null;
    _attributes: Map<string, any>;
    _changes: Map<string, any>;
    _relations: Map<string, ODataModelRelation<any>>;
    _resource: ODataResource<T> | null;
    _annotations: ODataEntityAnnotations<T> | null;
    _reset: boolean;
    _reparent: boolean;
    _silent: boolean;
    _meta: ODataModelOptions<any>;
    events$: EventEmitter<ODataModelEvent<T>>;
    constructor(data?: Partial<T> | {
        [name: string]: any;
    }, { parent, resource, annots, reset, }?: {
        parent?: [
            ODataModel<any> | ODataCollection<any, ODataModel<any>>,
            ODataModelField<any> | null
        ];
        resource?: ODataResource<T>;
        annots?: ODataEntityAnnotations<T>;
        reset?: boolean;
    });
    resource(): ODataEntityResource<T> | ODataNavigationPropertyResource<T> | ODataPropertyResource<T> | ODataSingletonResource<T>;
    navigationProperty<N>(name: keyof T | string): ODataNavigationPropertyResource<N>;
    property<N>(name: string): ODataPropertyResource<N>;
    attach(resource: ODataEntityResource<T> | ODataNavigationPropertyResource<T> | ODataPropertyResource<T> | ODataSingletonResource<T>): void;
    schema(): ODataStructuredType<any>;
    annots(): ODataEntityAnnotations<T>;
    key({ field_mapping, resolve, }?: {
        field_mapping?: boolean;
        resolve?: boolean;
    }): EntityKey<T> | {
        [name: string]: any;
    } | undefined;
    isNew(): boolean;
    isParentOf(child: ODataModel<any> | ODataCollection<any, ODataModel<any>>): boolean;
    referential(field: ODataModelField<any>, { field_mapping, resolve, }?: {
        field_mapping?: boolean;
        resolve?: boolean;
    }): {
        [name: string]: any;
    } | null | undefined;
    referenced(field: ODataModelField<any>, { field_mapping, resolve, }?: {
        field_mapping?: boolean;
        resolve?: boolean;
    }): {
        [name: string]: any;
    } | null | undefined;
    _errors?: {
        [key: string]: any;
    };
    validate({ method, navigation, }?: {
        method?: 'create' | 'update' | 'modify';
        navigation?: boolean;
    }): {
        [name: string]: string[];
    } | undefined;
    isValid({ method, navigation, }?: {
        method?: 'create' | 'update' | 'modify';
        navigation?: boolean;
    }): boolean;
    defaults(): any;
    toEntity({ client_id, include_navigation, include_concurrency, include_computed, include_key, include_non_field, changes_only, field_mapping, chain, }?: {
        client_id?: boolean;
        include_navigation?: boolean;
        include_concurrency?: boolean;
        include_computed?: boolean;
        include_key?: boolean;
        include_non_field?: boolean;
        changes_only?: boolean;
        field_mapping?: boolean;
        chain?: (ODataModel<any> | ODataCollection<any, ODataModel<any>>)[];
    }): T | {
        [name: string]: any;
    };
    toJSON(): T | {
        [name: string]: any;
    };
    attributes({ changes_only, include_concurrency, include_computed, include_non_field, field_mapping, }?: {
        changes_only?: boolean;
        include_concurrency?: boolean;
        include_computed?: boolean;
        include_non_field?: boolean;
        field_mapping?: boolean;
    }): {
        [name: string]: any;
    };
    set(path: string | string[], value: any): any;
    get(path: string | string[]): any;
    reset({ path, silent, }?: {
        path?: string | string[];
        silent?: boolean;
    }): void;
    clear({ silent }?: {
        silent?: boolean;
    }): void;
    assign(entity: Partial<T> | {
        [name: string]: any;
    }, { reset, reparent, silent, }?: {
        reset?: boolean;
        reparent?: boolean;
        silent?: boolean;
    }): void;
    clone<M extends ODataModel<T>>(): M;
    private _request;
    fetch({ ...options }?: ODataOptions & {
        options?: ODataOptions;
    }): Observable<this>;
    save({ method, navigation, validate, ...options }?: ODataOptions & {
        method?: 'create' | 'update' | 'modify';
        navigation?: boolean;
        validate?: boolean;
        options?: ODataOptions;
    }): Observable<this>;
    destroy({ ...options }?: ODataOptions & {
        options?: ODataOptions;
    }): Observable<this>;
    /**
     * Create an execution context for change the internal query of a resource
     * @param func Function to execute
     */
    query(func: (q: ODataQueryOptionsHandler<T>, s?: ODataStructuredType<T>) => void): this;
    /**
     * Perform a check on the internal state of the model and return true if the model is changed.
     * @param include_navigation Check in navigation properties
     * @returns true if the model has changed, false otherwise
     */
    hasChanged({ include_navigation, }?: {
        include_navigation?: boolean;
    }): boolean;
    /**
     * Create an execution context for a given function, where the model is bound to its entity endpoint
     * @param ctx Context function
     * @returns The result of the context
     */
    asEntity<R>(ctx: (model: this) => R): R;
    callFunction<P, R>(name: string, params: P | null, responseType: 'property' | 'model' | 'collection' | 'none', { ...options }?: {} & ODataFunctionOptions<R>): Observable<R | ODataModel<R> | ODataCollection<R, ODataModel<R>> | null>;
    callAction<P, R>(name: string, params: P | null, responseType: 'property' | 'model' | 'collection' | 'none', { ...options }?: {} & ODataActionOptions<R>): Observable<R | ODataModel<R> | ODataCollection<R, ODataModel<R>> | null>;
    cast<S>(type: string): ODataModel<S>;
    fetchNavigationProperty<S>(name: keyof T | string, responseType: 'model' | 'collection', { ...options }?: {} & ODataQueryArgumentsOptions<S>): Observable<ODataModel<S> | ODataCollection<S, ODataModel<S>> | null>;
    getValue<P>(name: keyof T | string, options?: ODataOptions): Observable<P | ODataModel<P> | ODataCollection<P, ODataModel<P>> | null>;
    setReference<N>(name: keyof T | string, model: ODataModel<N> | ODataCollection<N, ODataModel<N>> | null, options?: ODataOptions): Observable<this>;
    getReference<P>(name: keyof T | string): ODataModel<P> | ODataCollection<P, ODataModel<P>> | null;
    equals(other: ODataModel<T>): boolean;
    get [Symbol.toStringTag](): string;
    collection(): ODataCollection<T, ODataModel<T>> | undefined;
    next(): ODataModel<T> | undefined;
    prev(): ODataModel<T> | undefined;
}
export declare const RESERVED_FIELD_NAMES: string[];
