export declare type Unpacked<T> = T extends (infer U)[] ? U : T;
export declare type Select<T> = SelectType<T> | SelectType<T>[];
export declare type SelectType<T> = string | keyof T;
export declare type Filter<T> = FilterType | FilterType[];
export declare type FilterType = string | {
    [name: string]: any;
};
export declare enum StandardAggregateMethods {
    sum = "sum",
    min = "min",
    max = "max",
    average = "average",
    countdistinct = "countdistinct"
}
export declare type Aggregate = string | {
    [propertyName: string]: {
        with: StandardAggregateMethods;
        as: string;
    };
};
export declare type OrderBy<T> = OrderByType<T> | OrderByType<T>[];
export declare type OrderByType<T> = string | OrderByObject<T>;
export declare type OrderByObject<T> = keyof T | [keyof T | string, 'asc' | 'desc'];
export declare type Expand<T> = ExpandType<T> | ExpandType<T>[];
export declare type ExpandType<T> = string | ExpandObject<T>;
export declare type ExpandObject<T> = keyof T | NestedExpandOptions<T>;
export declare type NestedExpandOptions<T> = {
    [P in keyof T]?: ExpandOptions<Unpacked<T[P]>>;
};
export declare type ExpandOptions<T> = {
    select?: Select<T>;
    filter?: Filter<T>;
    orderBy?: OrderBy<T>;
    top?: number;
    skip?: number;
    levels?: number | 'max';
    count?: boolean | Filter<T>;
    expand?: Expand<T>;
};
export declare type Transform<T> = {
    aggregate?: Aggregate | Array<Aggregate>;
    filter?: Filter<T>;
    groupBy?: GroupBy<T>;
};
export declare type GroupBy<T> = {
    properties: Array<keyof T>;
    transform?: Transform<T>;
};
export declare enum QueryCustomTypes {
    Raw = 0,
    Alias = 1,
    Duration = 2,
    Binary = 3
}
export declare type QueryCustomType = {
    type: QueryCustomTypes;
    value: any;
    name?: string;
};
export declare type Value = string | Date | number | boolean | QueryCustomType;
export declare const raw: (value: string) => QueryCustomType;
export declare const alias: (value: any, name?: string) => QueryCustomType;
export declare const duration: (value: string) => QueryCustomType;
export declare const binary: (value: string) => QueryCustomType;
export declare const isQueryCustomType: (value: any) => boolean;
export declare type QueryOptions<T> = ExpandOptions<T> & {
    search: string;
    transform: {
        [name: string]: any;
    } | {
        [name: string]: any;
    }[];
    skip: number;
    skiptoken: string;
    key: string | number | {
        [name: string]: any;
    };
    count: boolean | Filter<T>;
    action: string;
    func: string | {
        [functionName: string]: {
            [parameterName: string]: any;
        };
    };
    format: string;
    aliases: QueryCustomType[];
    escape: boolean;
};
export declare const ITEM_ROOT = "";
export default function <T>({ select, search, skiptoken, format, top, skip, filter, transform, orderBy, key, count, expand, action, func, aliases, escape, }?: Partial<QueryOptions<T>>): string;
export declare function buildPathAndQuery<T>({ select, search, skiptoken, format, top, skip, filter, transform, orderBy, key, count, expand, action, func, aliases, escape, }?: Partial<QueryOptions<T>>): [string, {
    [name: string]: any;
}];
export declare function normalizeValue(value: Value, { aliases, escape, }?: {
    aliases?: QueryCustomType[];
    escape?: boolean;
}): any;
