import type { QueryCustomType } from '../builder';
export interface Renderable {
    render({ aliases, escape, prefix, }: {
        aliases?: QueryCustomType[];
        escape?: boolean;
        prefix?: string;
    }): string;
    toString(): string;
    toJSON(): any;
    clone(): any;
}
export declare class Field<T extends object> implements ProxyHandler<T> {
    name: string;
    constructor(name?: string);
    static factory<T extends object>(name?: string): T;
    get(target: T, key: string | symbol): any;
    has(target: T, key: string): any;
}
export declare function render(value: any, { aliases, normalize, escape, prefix, }?: {
    aliases?: QueryCustomType[];
    normalize?: boolean;
    escape?: boolean;
    prefix?: string;
}): string | number | boolean | null;
export declare class Function<T> implements Renderable {
    protected name: string;
    protected values: any[];
    protected normalize: boolean;
    protected escape: boolean;
    constructor(name: string, values: any[], normalize?: boolean, escape?: boolean);
    get [Symbol.toStringTag](): string;
    toJSON(): {
        $type: string;
        name: string;
        values: any[];
        normalize: boolean;
    };
    render({ aliases, escape, prefix, }: {
        aliases?: QueryCustomType[];
        escape?: boolean;
        prefix?: string;
    }): string;
    clone(): Function<T>;
}
export declare class StringAndCollectionFunctions<T> {
    concat(field: any, value: any, normalize?: boolean): Function<T>;
    contains(field: any, value: any, normalize?: boolean): Function<T>;
    endsWith(field: any, value: any, normalize?: boolean): Function<T>;
    indexOf(field: any, value: any, normalize?: boolean): Function<T>;
    length(value: any, normalize?: boolean): Function<T>;
    startsWith(field: any, value: any, normalize?: boolean): Function<T>;
    subString(field: any, start: number, length?: number): Function<T>;
}
export declare class CollectionFunctions<T> {
    hasSubset(s1: T, s2: any): Function<T>;
    hasSubsequence(s1: T, s2: any): Function<T>;
}
export declare class StringFunctions<T> {
    matchesPattern(value: T | string, pattern: string): Function<T>;
    toLower(value: T): Function<T>;
    toUpper(value: T): Function<T>;
    trim(value: T): Function<T>;
}
export declare class DateAndTimeFunctions<T> {
    date(value: any): Function<T>;
    day(value: any): Function<T>;
    fractionalseconds(value: any): Function<T>;
    hour(value: any): Function<T>;
    maxdatetime(value: any): Function<T>;
    mindatetime(value: any): Function<T>;
    minute(value: any): Function<T>;
    month(value: any): Function<T>;
    now(): Function<T>;
    second(value: any): Function<T>;
    time(value: any): Function<T>;
    totaloffsetminutes(value: any): Function<T>;
    totalseconds(value: any): Function<T>;
    year(value: any): Function<T>;
}
export declare class ArithmeticFunctions<T> {
    ceiling(value: T | string): Function<T>;
    floor(value: T | string): Function<T>;
    round(value: T | string): Function<T>;
}
export declare class TypeFunctions<T> {
    cast(value: T | string, type?: string): Function<T>;
    isof(value: T | string, type?: string): Function<T>;
}
export declare class GeoFunctions<T> {
    geoDistance(value: T, point: string, normalize?: boolean): Function<T>;
    geoIntersects(value: T, polygon: string, normalize?: boolean): Function<T>;
    geoLength(line: T, normalize?: boolean): Function<T>;
}
export declare class ConditionalFunctions<T> {
    case(condition: T | string, value: any): Function<T>;
}
export declare class Operator<T> implements Renderable {
    protected op: string;
    protected values: any[];
    protected normalize: boolean;
    constructor(op: string, values: any[], normalize?: boolean);
    get [Symbol.toStringTag](): string;
    toJSON(): {
        $type: string;
        op: string;
        values: any[];
        normalize: boolean;
    };
    render({ aliases, escape, prefix, }: {
        aliases?: QueryCustomType[];
        escape?: boolean;
        prefix?: string;
    }): string;
    clone(): Operator<unknown>;
}
export declare class LogicalOperators<T> {
    eq(left: any, right: any, normalize?: boolean): Operator<unknown>;
    ne(left: any, right: any, normalize?: boolean): Operator<unknown>;
    gt(left: any, right: any, normalize?: boolean): Operator<unknown>;
    ge(left: any, right: any, normalize?: boolean): Operator<unknown>;
    lt(left: any, right: any, normalize?: boolean): Operator<unknown>;
    le(left: any, right: any, normalize?: boolean): Operator<unknown>;
    not(value: any, normalize?: boolean): Operator<T>;
    has(left: any, right: any, normalize?: boolean): Operator<unknown>;
    in(left: any, right: any, normalize?: boolean): Operator<unknown>;
}
export declare class ArithmeticOperators<T> {
    add(left: any, right: any, normalize?: boolean): Operator<T>;
    sub(left: any, right: any, normalize?: boolean): Operator<unknown>;
    mul(left: any, right: any, normalize?: boolean): Operator<unknown>;
    div(left: any, right: any, normalize?: boolean): Operator<unknown>;
    mod(left: any, right: any, normalize?: boolean): Operator<unknown>;
    neg(value: any, normalize?: boolean): Operator<unknown>;
}
export declare class Grouping<T> implements Renderable {
    protected group: Renderable;
    constructor(group: Renderable);
    get [Symbol.toStringTag](): string;
    toJSON(): {
        $type: string;
        group: any;
    };
    render({ aliases, escape, prefix, }: {
        aliases?: QueryCustomType[];
        escape?: boolean;
        prefix?: string;
    }): string;
    clone(): Grouping<unknown>;
}
export declare class Lambda<T> implements Renderable {
    protected op: string;
    protected values: any[];
    protected alias?: string | undefined;
    constructor(op: string, values: any[], alias?: string | undefined);
    get [Symbol.toStringTag](): string;
    toJSON(): {
        $type: string;
        op: string;
        values: any[];
        alias: string | undefined;
    };
    render({ aliases, escape, prefix, }: {
        aliases?: QueryCustomType[];
        escape?: boolean;
        prefix?: string;
    }): string;
    clone(): Lambda<unknown>;
}
export declare class LambdaOperators<T> {
    any(field: T, value: any, alias?: string): Lambda<unknown>;
    all(field: T, value: any, alias?: string): Lambda<unknown>;
}
export declare class ODataOperators<T> {
}
export interface ODataOperators<T> extends LogicalOperators<T>, ArithmeticOperators<T>, LambdaOperators<T> {
}
export declare const operators: ODataOperators<any>;
export declare class ODataFunctions<T> {
}
export interface ODataFunctions<T> extends StringAndCollectionFunctions<T>, CollectionFunctions<T>, StringFunctions<T>, DateAndTimeFunctions<T>, ArithmeticFunctions<T>, TypeFunctions<T>, GeoFunctions<T>, ConditionalFunctions<T> {
}
export declare const functions: ODataFunctions<any>;
export declare class ODataSyntax<T> {
}
export interface ODataSyntax<T> extends ODataOperators<T>, ODataFunctions<T> {
}
export declare const syntax: ODataSyntax<any>;
