import { QueryOptionNames } from '../../types';
import { Expand, Filter, OrderBy, Select, Transform } from './builder';
import { ComputeExpression, Expression, FilterExpression, OrderByExpression, SearchExpression } from './expressions';
import { ExpandExpression } from './expressions/expand';
import { SelectExpression } from './expressions/select';
import { ODataQueryOptionHandler } from './handlers';
export declare type ODataQueryArguments<T> = {
    [QueryOptionNames.select]?: Select<T> | SelectExpression<T> | null;
    [QueryOptionNames.expand]?: Expand<T> | ExpandExpression<T> | null;
    [QueryOptionNames.compute]?: string | ComputeExpression<T> | null;
    [QueryOptionNames.filter]?: Filter<T> | FilterExpression<T> | null;
    [QueryOptionNames.search]?: string | SearchExpression<T> | null;
    [QueryOptionNames.transform]?: Transform<T> | null;
    [QueryOptionNames.orderBy]?: OrderBy<T> | OrderByExpression<T> | null;
    [QueryOptionNames.top]?: number | null;
    [QueryOptionNames.skip]?: number | null;
    [QueryOptionNames.skiptoken]?: string | null;
    [QueryOptionNames.format]?: string | null;
    [QueryOptionNames.levels]?: number | 'max' | null;
    [QueryOptionNames.count]?: boolean | null;
};
export declare class ODataQueryOptions<T> {
    values: Map<QueryOptionNames, any>;
    constructor(values?: Map<QueryOptionNames, any> | {
        [name: string]: any;
    });
    pathAndParams(escape?: boolean): [string, {
        [name: string]: any;
    }];
    toString(): string;
    toJSON(): {};
    toQueryArguments(): ODataQueryArguments<T>;
    clone<O>(): ODataQueryOptions<O>;
    expression(key: QueryOptionNames, exp?: Expression<T>): any;
    option<O>(key: QueryOptionNames, opts?: O): ODataQueryOptionHandler<O>;
    has(key: QueryOptionNames): boolean;
    remove(...keys: QueryOptionNames[]): void;
    keep(...keys: QueryOptionNames[]): void;
    clear(): void;
}
