import { Observable } from 'rxjs';
export declare type ODataVersion = '2.0' | '3.0' | '4.0';
export declare type FetchPolicy = 'cache-first' | 'cache-and-network' | 'network-only' | 'no-cache' | 'cache-only';
export declare type ODataMetadataType = 'minimal' | 'full' | 'none';
export declare type CacheCacheability = 'public' | 'private' | 'no-cache' | 'no-store';
export declare enum PathSegmentNames {
    batch = "batch",
    metadata = "metadata",
    entitySet = "entitySet",
    singleton = "singleton",
    type = "type",
    property = "property",
    navigationProperty = "navigationProperty",
    reference = "reference",
    value = "value",
    count = "count",
    function = "function",
    action = "action"
}
export declare enum QueryOptionNames {
    select = "select",
    expand = "expand",
    compute = "compute",
    filter = "filter",
    search = "search",
    transform = "transform",
    orderBy = "orderBy",
    top = "top",
    skip = "skip",
    skiptoken = "skiptoken",
    format = "format",
    levels = "levels",
    count = "count"
}
export interface ApiOptions {
    version?: ODataVersion;
    params?: {
        [param: string]: string | string[];
    };
    headers?: {
        [param: string]: string | string[];
    };
    withCredentials?: boolean;
    accept?: {
        exponentialDecimals?: boolean;
        metadata?: ODataMetadataType;
        ieee754Compatible?: boolean;
        streaming?: boolean;
    };
    etag?: {
        ifMatch?: boolean;
        ifNoneMatch?: boolean;
    };
    prefer?: {
        maxPageSize?: number;
        return?: 'representation' | 'minimal';
        continueOnError?: boolean;
        includeAnnotations?: string;
    };
    stripMetadata?: ODataMetadataType;
    fetchPolicy?: FetchPolicy;
    bodyQueryOptions?: QueryOptionNames[];
    stringAsEnum?: boolean;
    deleteRefBy?: 'path' | 'id';
    nonParenthesisForEmptyParameterFunction?: boolean;
}
export interface ParserOptions {
    version?: ODataVersion;
    exponentialDecimals?: boolean;
    metadata?: ODataMetadataType;
    ieee754Compatible?: boolean;
    streaming?: boolean;
    stringAsEnum?: boolean;
    deleteRefBy?: 'path' | 'id';
    nonParenthesisForEmptyParameterFunction?: boolean;
}
export interface ResponseOptions extends ParserOptions {
    cacheability?: CacheCacheability;
    maxAge?: number;
}
export interface StructuredTypeFieldOptions extends ParserOptions {
    field: StructuredTypeFieldConfig;
}
export interface Parser<T> {
    deserialize(value: any, options?: ParserOptions | StructuredTypeFieldOptions): T;
    serialize(value: any, options?: ParserOptions | StructuredTypeFieldOptions): any;
    encode(value: any, options?: ParserOptions | StructuredTypeFieldOptions): any;
}
export declare const NONE_PARSER: Parser<any>;
export interface Cache {
    put<T>(key: string, payload: T, ...opts: any[]): void;
    get<T>(key: string, ...opts: any[]): T | undefined;
}
export declare type ApiConfig = {
    serviceRootUrl: string;
    name?: string;
    version?: ODataVersion;
    default?: boolean;
    creation?: Date;
    cache?: Cache;
    errorHandler?: (error: any, caught: Observable<any>) => Observable<never>;
    options?: ApiOptions;
    parsers?: {
        [type: string]: Parser<any>;
    };
    schemas?: SchemaConfig[];
};
export declare type AnnotationConfig = {
    term: string;
    string?: string;
    bool?: boolean;
    int?: number;
    permissions?: string[];
    properties?: string[];
};
export declare type SchemaConfig = {
    namespace: string;
    alias?: string;
    annotations?: AnnotationConfig[];
    enums?: EnumTypeConfig<any>[];
    entities?: StructuredTypeConfig<any>[];
    callables?: CallableConfig[];
    containers?: EntityContainerConfig[];
};
export declare type EntityContainerConfig = {
    name: string;
    annotations?: AnnotationConfig[];
    entitySets?: EntitySetConfig[];
};
export declare type EnumTypeFieldConfig<E> = {
    value: E;
    annotations?: AnnotationConfig[];
};
export declare type EnumTypeConfig<E> = {
    name: string;
    flags?: boolean;
    annotations?: AnnotationConfig[];
    members: {
        [name: string]: E;
    } | {
        [value: number]: string;
    };
    fields: {
        [member: string]: EnumTypeFieldConfig<E>;
    };
};
export declare type StructuredTypeFieldConfig = {
    type: string;
    default?: any;
    maxLength?: number;
    key?: boolean;
    collection?: boolean;
    nullable?: boolean;
    navigation?: boolean;
    precision?: number;
    annotations?: AnnotationConfig[];
    scale?: number | 'variable';
    referentials?: {
        property: string;
        referencedProperty: string;
    }[];
    referential?: string;
    referenced?: string;
};
export declare type StructuredTypeConfig<T> = {
    name: string;
    base?: string;
    open?: boolean;
    model?: {
        new (...params: any[]): any;
    };
    collection?: {
        new (...params: any[]): any;
    };
    annotations?: AnnotationConfig[];
    keys?: {
        name: string;
        alias?: string;
    }[];
    fields: {
        [P in keyof T]?: StructuredTypeFieldConfig;
    };
};
export declare type Parameter = {
    type: string;
    nullable?: boolean;
    collection?: boolean;
};
export declare type CallableConfig = {
    name: string;
    entitySetPath?: string;
    bound?: boolean;
    composable?: boolean;
    parameters?: {
        [name: string]: Parameter;
    };
    return?: {
        type: string;
        collection?: boolean;
    };
};
export declare type EntitySetConfig = {
    name: string;
    entityType: string;
    service: {
        new (...params: any[]): any;
    };
    annotations?: AnnotationConfig[];
};
